@@ -1,5 +1,5 @@
 /* dirname -- strip filename suffix from pathname
-   Copyright (C) 90, 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1990-1997, 1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,12 +25,17 @@
 #include "long-options.h"
 #include "error.h"
 
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "dirname"
+
+#define AUTHORS "David MacKenzie and Jim Meyering"
+
 void strip_trailing_slashes ();
 
 /* The name this program was run with. */
 char *program_name;
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -50,7 +55,7 @@ output `.' (meaning the current directory).\n\
   --help      display this help and exit\n\
   --version   output version information and exit\n\
 "));
-      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
     }
   exit (status);
 }
@@ -66,7 +71,15 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  parse_long_options (argc, argv, "dirname", GNU_PACKAGE, VERSION, usage);
+  parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+		      AUTHORS, usage);
+  /* The above handles --help and --version.
+     Since there is no other invocation of getopt, handle `--' here.  */
+  if (argc > 1 && STREQ (argv[1], "--"))
+    {
+      --argc;
+      ++argv;
+    }
 
   if (argc != 2)
     {
