@@ -1,5 +1,5 @@
 /* hostname - set or print the name of current host system
-   Copyright (C) 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1994-1997, 1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,10 +25,14 @@
 #include "long-options.h"
 #include "error.h"
 
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "hostname"
+
+#define AUTHORS "Jim Meyering"
+
 #if !defined(HAVE_SETHOSTNAME) && defined(HAVE_SYSINFO) && \
      defined (HAVE_SYS_SYSTEMINFO_H) && defined(HAVE_LIMITS_H)
-#include <limits.h>
-#include <sys/systeminfo.h>
+# include <sys/systeminfo.h>
 
 int
 sethostname (name, namelen)
@@ -43,7 +47,7 @@ sethostname (name, namelen)
   return (result == -1 ? result : 0);
 }
 
-#define HAVE_SETHOSTNAME 1  /* Now we have it... */
+# define HAVE_SETHOSTNAME 1  /* Now we have it... */
 #endif
 
 char *xgethostname ();
@@ -51,7 +55,7 @@ char *xgethostname ();
 /* The name this program was run with. */
 char *program_name;
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -68,7 +72,7 @@ Print the hostname of the current system.\n\
   --version   output version information and exit\n\
 ")
              , program_name, program_name);
-      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
     }
   exit (status);
 }
@@ -83,7 +87,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  parse_long_options (argc, argv, "hostname", GNU_PACKAGE, VERSION, usage);
+  parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+		      AUTHORS, usage);
 
 #ifdef HAVE_SETHOSTNAME
   if (argc == 2)
@@ -93,7 +98,7 @@ main (int argc, char **argv)
       /* Set hostname to argv[1].  */
       err = sethostname (argv[1], strlen (argv[1]));
       if (err != 0)
-	error (1, errno, "%s", argv[1]);
+	error (1, errno, _("cannot set hostname to `%s'"), argv[1]);
       exit (0);
     }
 #else
