@@ -1,5 +1,5 @@
 /* nice -- run a program with modified scheduling priority
-   Copyright (C) 90, 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1990-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,28 +20,31 @@
 #include <config.h>
 #include <stdio.h>
 
-#define NDEBUG
 #include <assert.h>
 
 #include <getopt.h>
 #include <sys/types.h>
 #ifndef NICE_PRIORITY
-#include <sys/time.h>
-#include <sys/resource.h>
+# include <sys/time.h>
+# include <sys/resource.h>
 #endif
 
 #include "system.h"
 #include "long-options.h"
 #include "error.h"
 
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "nice"
+
+#define AUTHORS "David MacKenzie"
+
 #ifdef NICE_PRIORITY
-#define GET_PRIORITY() nice (0)
+# define GET_PRIORITY() nice (0)
 #else
-#define GET_PRIORITY() getpriority (PRIO_PROCESS, 0)
+# define GET_PRIORITY() getpriority (PRIO_PROCESS, 0)
 #endif
 
-static int isinteger __P ((char *s));
-static void usage __P ((int status));
+static int isinteger PARAMS ((char *s));
 
 /* The name this program was run with. */
 char *program_name;
@@ -52,6 +55,29 @@ static struct option const longopts[] =
   {NULL, 0, NULL, 0}
 };
 
+void
+usage (int status)
+{
+  if (status != 0)
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+	     program_name);
+  else
+    {
+      printf (_("Usage: %s [OPTION]... [COMMAND [ARG]...]\n"), program_name);
+      printf (_("\
+Run COMMAND with an adjusted scheduling priority.\n\
+With no COMMAND, print the current scheduling priority.  ADJUST is 10\n\
+by default.  Range goes from -20 (highest priority) to 19 (lowest).\n\
+\n\
+  -ADJUST                   increment priority by ADJUST first\n\
+  -n, --adjustment=ADJUST   same as -ADJUST\n\
+      --help                display this help and exit\n\
+      --version             output version information and exit\n"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
+    }
+  exit (status);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -66,7 +92,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  parse_long_options (argc, argv, "nice", GNU_PACKAGE, VERSION, usage);
+  parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+		      AUTHORS, usage);
 
   for (i = 1; i < argc; /* empty */)
     {
@@ -106,7 +133,7 @@ main (int argc, char **argv)
 	  optind = 0;
 
 	  if ((optc = getopt_long (argc - (i - 1), fake_argv, "+n:",
-				   longopts, (int *) 0)) != EOF)
+				   longopts, NULL)) != -1)
 	    {
 	      switch (optc)
 		{
@@ -186,26 +213,3 @@ isinteger (char *s)
     }
   return 1;
 }
-
-static void
-usage (int status)
-{
-  if (status != 0)
-    fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
-  else
-    {
-      printf (_("Usage: %s [OPTION]... [COMMAND [ARG]...]\n"), program_name);
-      printf (_("\
-Run COMMAND with an adjusted scheduling priority.\n\
-With no COMMAND, print the current scheduling priority.  ADJUST is 10\n\
-by default.  Range goes from -20 (highest priority) to 19 (lowest).\n\
-\n\
-  -ADJUST                   increment priority by ADJUST first\n\
-  -n, --adjustment=ADJUST   same as -ADJUST\n\
-      --help                display this help and exit\n\
-      --version             output version information and exit\n"));
-      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
-    }
-  exit (status);
-}
