@@ -1,5 +1,5 @@
 /* su for GNU.  Run a shell with substitute user and group IDs.
-   Copyright (C) 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1992-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -75,29 +75,29 @@
 #include <sys/types.h>
 #include <pwd.h>
 #include <grp.h>
+
+/* Hide any system prototype for getusershell.
+   This is necessary because some Cray systems have a conflicting
+   prototype (returning `int') in <unistd.h>.  */
+#define getusershell _getusershell_sys_proto_
+
 #include "system.h"
 
-#if defined(HAVE_SYSLOG_H) && defined(HAVE_SYSLOG)
-#include <syslog.h>
-#else /* !HAVE_SYSLOG_H */
-#ifdef SYSLOG_SUCCESS
-#undef SYSLOG_SUCCESS
-#endif
-#ifdef SYSLOG_FAILURE
-#undef SYSLOG_FAILURE
-#endif
-#ifdef SYSLOG_NON_ROOT
-#undef SYSLOG_NON_ROOT
-#endif
-#endif /* !HAVE_SYSLOG_H */
+#undef getusershell
 
-#ifdef _POSIX_VERSION
-#include <limits.h>
-#else /* not _POSIX_VERSION */
+#if HAVE_SYSLOG_H && HAVE_SYSLOG
+# include <syslog.h>
+#else
+# undef SYSLOG_SUCCESS
+# undef SYSLOG_FAILURE
+# undef SYSLOG_NON_ROOT
+#endif
+
+#ifndef _POSIX_VERSION
 struct passwd *getpwuid ();
 struct group *getgrgid ();
 uid_t getuid ();
-#include <sys/param.h>
+# include <sys/param.h>
 #endif /* not _POSIX_VERSION */
 
 #ifndef HAVE_ENDGRENT
@@ -108,28 +108,33 @@ uid_t getuid ();
 # define endpwent() ((void) 0)
 #endif
 
-#ifdef HAVE_SHADOW_H
-#include <shadow.h>
+#if HAVE_SHADOW_H
+# include <shadow.h>
 #endif
 
 #include "error.h"
 
-#ifdef HAVE_PATHS_H
-#include <paths.h>
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "su"
+
+#define AUTHORS "David MacKenzie"
+
+#if HAVE_PATHS_H
+# include <paths.h>
 #endif
 
 /* The default PATH for simulated logins to non-superuser accounts.  */
 #ifdef _PATH_DEFPATH
-#define DEFAULT_LOGIN_PATH _PATH_DEFPATH
+# define DEFAULT_LOGIN_PATH _PATH_DEFPATH
 #else
-#define DEFAULT_LOGIN_PATH ":/usr/ucb:/bin:/usr/bin"
+# define DEFAULT_LOGIN_PATH ":/usr/ucb:/bin:/usr/bin"
 #endif
 
 /* The default PATH for simulated logins to superuser accounts.  */
 #ifdef _PATH_DEFPATH_ROOT
-#define DEFAULT_ROOT_LOGIN_PATH _PATH_DEFPATH_ROOT
+# define DEFAULT_ROOT_LOGIN_PATH _PATH_DEFPATH_ROOT
 #else
-#define DEFAULT_ROOT_LOGIN_PATH "/usr/ucb:/bin:/usr/bin:/etc"
+# define DEFAULT_ROOT_LOGIN_PATH "/usr/ucb:/bin:/usr/bin:/etc"
 #endif
 
 /* The shell to run if none is given in the user's passwd entry.  */
@@ -144,9 +149,7 @@ char *getusershell ();
 void endusershell ();
 void setusershell ();
 
-char *basename ();
-char *xmalloc ();
-char *xrealloc ();
+char *base_name ();
 char *xstrdup ();
 
 extern char **environ;
@@ -154,12 +157,6 @@ extern char **environ;
 /* The name this program was run with.  */
 char *program_name;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output and exit.  */
-static int show_version;
-
 /* If nonzero, pass the `-f' option to the subshell.  */
 static int fast_startup;
 
@@ -172,12 +169,12 @@ static int change_environment;
 static struct option const longopts[] =
 {
   {"command", required_argument, 0, 'c'},
-  {"fast", no_argument, &fast_startup, 1},
-  {"help", no_argument, &show_help, 1},
-  {"login", no_argument, &simulate_login, 1},
+  {"fast", no_argument, NULL, 'f'},
+  {"login", no_argument, NULL, 'l'},
   {"preserve-environment", no_argument, &change_environment, 0},
   {"shell", required_argument, 0, 's'},
-  {"version", no_argument, &show_version, 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {0, 0, 0, 0}
 };
 
@@ -228,35 +225,40 @@ log_su (const struct passwd *pw, int successful)
 {
   const char *new_user, *old_user, *tty;
 
-#ifndef SYSLOG_NON_ROOT
+# ifndef SYSLOG_NON_ROOT
   if (pw->pw_uid)
     return;
-#endif
+# endif
   new_user = pw->pw_name;
   /* The utmp entry (via getlogin) is probably the best way to identify
      the user, especially if someone su's from a su-shell.  */
   old_user = getlogin ();
   if (old_user == NULL)
-    old_user = "";
+    {
+      /* getlogin can fail -- usually due to lack of utmp entry.
+	 Resort to getpwuid.  */
+      struct passwd *pwd = getpwuid (getuid ());
+      old_user = (pwd ? pwd->pw_name : "");
+    }
   tty = ttyname (2);
   if (tty == NULL)
-    tty = "";
+    tty = "none";
   /* 4.2BSD openlog doesn't have the third parameter.  */
-  openlog (basename (program_name), 0
-#ifdef LOG_AUTH
+  openlog (base_name (program_name), 0
+# ifdef LOG_AUTH
 	   , LOG_AUTH
-#endif
+# endif
 	   );
   syslog (LOG_NOTICE,
-#ifdef SYSLOG_NON_ROOT
+# ifdef SYSLOG_NON_ROOT
 	  "%s(to %s) %s on %s",
-#else
+# else
 	  "%s%s on %s",
-#endif
+# endif
 	  successful ? "" : "FAILED SU ",
-#ifdef SYSLOG_NON_ROOT
+# ifdef SYSLOG_NON_ROOT
 	  new_user,
-#endif
+# endif
 	  old_user, tty);
   closelog ();
 }
@@ -280,7 +282,7 @@ correct_password (const struct passwd *pw)
     correct = sp->sp_pwdp;
   else
 #endif
-  correct = pw->pw_passwd;
+    correct = pw->pw_passwd;
 
   if (getuid () == 0 || correct == 0 || correct[0] == '\0')
     return 1;
@@ -317,8 +319,9 @@ modify_environment (const struct passwd *pw, const char *shell)
       xputenv (concat ("SHELL", "=", shell));
       xputenv (concat ("USER", "=", pw->pw_name));
       xputenv (concat ("LOGNAME", "=", pw->pw_name));
-      xputenv (concat ("PATH", "=", pw->pw_uid
-		       ? DEFAULT_LOGIN_PATH : DEFAULT_ROOT_LOGIN_PATH));
+      xputenv (concat ("PATH", "=", (pw->pw_uid
+				     ? DEFAULT_LOGIN_PATH
+				     : DEFAULT_ROOT_LOGIN_PATH)));
     }
   else
     {
@@ -375,14 +378,14 @@ run_shell (const char *shell, const char *command, char **additional_args)
       char *arg0;
       char *shell_basename;
 
-      shell_basename = basename (shell);
+      shell_basename = base_name (shell);
       arg0 = xmalloc (strlen (shell_basename) + 2);
       arg0[0] = '-';
       strcpy (arg0 + 1, shell_basename);
       args[0] = arg0;
     }
   else
-    args[0] = basename (shell);
+    args[0] = base_name (shell);
   if (fast_startup)
     args[argno++] = "-f";
   if (command)
@@ -419,7 +422,7 @@ restricted_shell (const char *shell)
   return 1;
 }
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -442,7 +445,7 @@ Change the effective user id and group id to that of USER.\n\
 \n\
 A mere - implies -l.   If USER not given, assume root.\n\
 "));
-      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
     }
   exit (status);
 }
@@ -467,8 +470,7 @@ main (int argc, char **argv)
   simulate_login = 0;
   change_environment = 1;
 
-  while ((optc = getopt_long (argc, argv, "c:flmps:", longopts, (int *) 0))
-	 != EOF)
+  while ((optc = getopt_long (argc, argv, "c:flmps:", longopts, NULL)) != -1)
     {
       switch (optc)
 	{
@@ -496,20 +498,15 @@ main (int argc, char **argv)
 	  shell = optarg;
 	  break;
 
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
 	default:
 	  usage (1);
 	}
     }
 
-  if (show_version)
-    {
-      printf ("su (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (0);
-    }
-
-  if (show_help)
-    usage (0);
-
   if (optind < argc && !strcmp (argv[optind], "-"))
     {
       simulate_login = 1;
@@ -525,6 +522,12 @@ main (int argc, char **argv)
     error (1, 0, _("user %s does not exist"), new_user);
   endpwent ();
 
+  /* Make sure pw->pw_shell is non-NULL.  It may be NULL when NEW_USER
+     is a username that is retrieved via NIS (YP), but that doesn't have
+     a default shell listed.  */
+  if (pw->pw_shell == NULL || pw->pw_shell[0] == '\0')
+    pw->pw_shell = (char *) DEFAULT_SHELL;
+
   /* Make a copy of the password information and point pw at the local
      copy instead.  Otherwise, some systems (e.g. Linux) would clobber
      the static data through the getlogin call from log_su.  */
@@ -548,8 +551,6 @@ main (int argc, char **argv)
     }
 #endif
 
-  if (pw->pw_shell == 0 || pw->pw_shell[0] == 0)
-    pw->pw_shell = (char *) DEFAULT_SHELL;
   if (shell == 0 && change_environment == 0)
     shell = getenv ("SHELL");
   if (shell != 0 && getuid () && restricted_shell (pw->pw_shell))
