@@ -1,5 +1,5 @@
 /* tee - read from standard input and write to standard output and files.
-   Copyright (C) 85, 90, 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 85,1990-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,12 +24,17 @@
 #include <getopt.h>
 
 #include "system.h"
+#include "closeout.h"
 #include "error.h"
 
-char *xmalloc ();
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "tee"
+
+#define AUTHORS "Mike Parker, Richard M. Stallman, and David MacKenzie"
+
 int full_write ();
 
-static int tee __P ((int nfiles, const char **files));
+static int tee PARAMS ((int nfiles, const char **files));
 
 /* If nonzero, append to output files rather than truncating them. */
 static int append;
@@ -40,22 +45,16 @@ static int ignore_interrupts;
 /* The name that this program was run with. */
 char *program_name;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output and exit.  */
-static int show_version;
-
 static struct option const long_options[] =
 {
   {"append", no_argument, NULL, 'a'},
-  {"help", no_argument, &show_help, 1},
   {"ignore-interrupts", no_argument, NULL, 'i'},
-  {"version", no_argument, &show_version, 1},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -72,7 +71,7 @@ Copy standard input to each FILE, and also to standard output.\n\
       --help                display this help and exit\n\
       --version             output version information and exit\n\
 "));
-      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
     }
   exit (status);
 }
@@ -91,8 +90,7 @@ main (int argc, char **argv)
   append = 0;
   ignore_interrupts = 0;
 
-  while ((optc = getopt_long (argc, argv, "ai", long_options, (int *) 0))
-	 != EOF)
+  while ((optc = getopt_long (argc, argv, "ai", long_options, NULL)) != -1)
     {
       switch (optc)
 	{
@@ -107,20 +105,15 @@ main (int argc, char **argv)
 	  ignore_interrupts = 1;
 	  break;
 
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
 	default:
 	  usage (1);
 	}
     }
 
-  if (show_version)
-    {
-      printf ("tee (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (0);
-    }
-
-  if (show_help)
-    usage (0);
-
   if (ignore_interrupts)
     {
 #ifdef _POSIX_SOURCE
@@ -150,11 +143,15 @@ main (int argc, char **argv)
   signal (SIGPIPE, SIG_IGN);
 #endif
 
+  /* FIXME: warn if tee is given no file arguments.
+     In that case it's just a slow imitation of `cat.'  */
+
   errs = tee (argc - optind, (const char **) &argv[optind]);
   if (close (0) != 0)
     error (1, errno, _("standard input"));
-  if (close (1) != 0)
-    error (1, errno, _("standard output"));
+
+  close_stdout ();
+
   exit (errs);
 }
 
@@ -165,17 +162,13 @@ main (int argc, char **argv)
 static int
 tee (int nfiles, const char **files)
 {
-  int *descriptors;
+  FILE **descriptors;
   char buffer[BUFSIZ];
-  register int bytes_read, i, ret = 0, mode;
+  int bytes_read, i;
+  int ret = 0;
+  const char *mode_string = (append ? "a" : "w");
 
-  descriptors = (int *) xmalloc ((nfiles + 1) * sizeof (int));
-
-  mode = O_WRONLY | O_CREAT;
-  if (append)
-    mode |= O_APPEND;
-  else
-    mode |= O_TRUNC;
+  descriptors = (FILE **) xmalloc ((nfiles + 1) * sizeof (descriptors[0]));
 
   /* Move all the names `up' one in the argv array to make room for
      the entry for standard output.  This writes into argv[argc].  */
@@ -184,17 +177,22 @@ tee (int nfiles, const char **files)
 
   /* In the array of NFILES + 1 descriptors, make
      the first one correspond to standard output.   */
-  descriptors[0] = 1;
+  descriptors[0] = stdout;
   files[0] = _("standard output");
+  SETVBUF (stdout, NULL, _IONBF, 0);
 
   for (i = 1; i <= nfiles; i++)
     {
-      descriptors[i] = open (files[i], mode, 0666);
-      if (descriptors[i] == -1)
+      descriptors[i] = fopen (files[i], mode_string);
+      if (descriptors[i] == NULL)
 	{
 	  error (0, errno, "%s", files[i]);
 	  ret = 1;
 	}
+      else
+	{
+	  SETVBUF (descriptors[i], NULL, _IONBF, 0);
+	}
     }
 
   while (1)
@@ -211,16 +209,8 @@ tee (int nfiles, const char **files)
 	 Standard output is the first one.  */
       for (i = 0; i <= nfiles; i++)
 	{
-	  if (descriptors[i] != -1
-	      && full_write (descriptors[i], buffer, bytes_read) < 0)
-	    {
-	      error (0, errno, "%s", files[i]);
-	      /* Don't close stdout.  That's done in main.  */
-	      if (descriptors[i] != 1)
-		close (descriptors[i]);
-	      descriptors[i] = -1;
-	      ret = 1;
-	    }
+	  if (descriptors[i] != NULL)
+	    fwrite (buffer, bytes_read, 1, descriptors[i]);
 	}
     }
 
@@ -232,7 +222,8 @@ tee (int nfiles, const char **files)
 
   /* Close the files, but not standard output.  */
   for (i = 1; i <= nfiles; i++)
-    if (descriptors[i] != -1 && close (descriptors[i]) != 0)
+    if (descriptors[i] != NULL
+	&& (ferror (descriptors[i]) || fclose (descriptors[i]) == EOF))
       {
 	error (0, errno, "%s", files[i]);
 	ret = 1;
