@@ -2,7 +2,7 @@
 
 /* Modified to run with the GNU shell by bfox. */
 
-/* Copyright (C) 1987-1993, 1994, 95, 1996 Free Software Foundation, Inc.
+/* Copyright (C) 1987-1999 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -28,32 +28,35 @@
 #include <stdio.h>
 #include <sys/types.h>
 
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "test"
+
 #define TEST_STANDALONE 1
 
 #if !defined (TEST_STANDALONE)
-#  include "shell.h"
-#  include "posixstat.h"
-#  include "filecntl.h"
+# include "shell.h"
+# include "posixstat.h"
+# include "filecntl.h"
 #else /* TEST_STANDALONE */
-#  include "system.h"
-#  include "group-member.h"
-#  include "error.h"
-#  if !defined (S_IXUGO)
-#    define S_IXUGO 0111
-#  endif /* S_IXUGO */
-#  if defined (_POSIX_VERSION)
-#    include <limits.h>
-#  else /* !_POSIX_VERSION */
-#    include <sys/param.h>
-#  endif /* _POSIX_VERSION */
-#define whitespace(c) (((c) == ' ') || ((c) == '\t'))
-#define digit(c)  ((c) >= '0' && (c) <= '9')
-#define digit_value(c) ((c) - '0')
+# include "system.h"
+# include "group-member.h"
+# include "error.h"
+# if !defined (S_IXUGO)
+#  define S_IXUGO 0111
+# endif /* S_IXUGO */
+# if defined (_POSIX_VERSION)
+#  include <limits.h>
+# else /* !_POSIX_VERSION */
+#  include <sys/param.h>
+# endif /* _POSIX_VERSION */
+# define whitespace(c) (((c) == ' ') || ((c) == '\t'))
+# define digit(c)  ((c) >= '0' && (c) <= '9')
+# define digit_value(c) ((c) - '0')
 char *program_name;
 #endif /* TEST_STANDALONE */
 
 #if !defined (_POSIX_VERSION)
-#  include <sys/file.h>
+# include <sys/file.h>
 #endif /* !_POSIX_VERSION */
 
 #include <errno.h>
@@ -61,26 +64,25 @@ char *program_name;
 extern int errno;
 #endif
 
-#if !defined (STREQ)
-#  define STREQ(a, b) ((a)[0] == (b)[0] && strcmp (a, b) == 0)
-#endif /* !STREQ */
+#undef STREQ
+#define STREQ(a, b) ((a)[0] == (b)[0] && strcmp (a, b) == 0)
 
 #if !defined (member)
-#  define member(c, s) ((c) ? (strchr ((s), (c)) ? 1 : 0) : 0)
+# define member(c, s) ((c) ? (strchr ((s), (c)) ? 1 : 0) : 0)
 #endif /* !member */
 
 extern gid_t getegid ();
 extern uid_t geteuid ();
 
 #if !defined (R_OK)
-#define R_OK 4
-#define W_OK 2
-#define X_OK 1
-#define F_OK 0
+# define R_OK 4
+# define W_OK 2
+# define X_OK 1
+# define F_OK 0
 #endif /* R_OK */
 
 /* This name is used solely when printing --version information.  */
-#define COMMAND_NAME "test"
+#define PROGRAM_NAME "test"
 
 /* The following few defines control the truth and false output of each stage.
    TRUE and FALSE are what we use to compute the final output value.
@@ -96,41 +98,39 @@ extern uid_t geteuid ();
 #define TRUTH_AND(a, b) ((a) & (b))
 
 #if defined (TEST_STANDALONE)
-#  define test_exit(val) exit (val)
+# define test_exit(val) exit (val)
 #else
    static jmp_buf test_exit_buf;
    static int test_error_return = 0;
-#  define test_exit(val) test_error_return = val, longjmp (test_exit_buf, 1)
+# define test_exit(val) test_error_return = val, longjmp (test_exit_buf, 1)
 #endif /* !TEST_STANDALONE */
 
-char *xrealloc ();
-
 static int pos;		/* The offset of the current argument in ARGV. */
 static int argc;	/* The number of arguments present in ARGV. */
 static char **argv;	/* The argument list. */
 
-static int unop __P ((int op));
-static int binop __P ((char *s));
-static int unary_operator __P ((void));
-static int binary_operator __P ((void));
-static int two_arguments __P ((void));
-static int three_arguments __P ((void));
-static int posixtest __P ((void));
+static int unop PARAMS ((int op));
+static int binop PARAMS ((char *s));
+static int unary_operator PARAMS ((void));
+static int binary_operator PARAMS ((void));
+static int two_arguments PARAMS ((void));
+static int three_arguments PARAMS ((void));
+static int posixtest PARAMS ((void));
 
-static int expr __P ((void));
-static int term __P ((void));
-static int and __P ((void));
-static int or __P ((void));
+static int expr PARAMS ((void));
+static int term PARAMS ((void));
+static int and PARAMS ((void));
+static int or PARAMS ((void));
 
 #if __GNUC__ >= 2 && defined (__GNUC_MINOR__) \
     && __GNUC_MINOR__ >= 5 && !defined (__STRICT_ANSI__)
-#define NO_RETURN_ATTRIBUTE __attribute__ ((noreturn))
+# define NO_RETURN_ATTRIBUTE __attribute__ ((noreturn))
 #else
-#define NO_RETURN_ATTRIBUTE /* empty */
+# define NO_RETURN_ATTRIBUTE /* empty */
 #endif
 
-static void test_syntax_error __P ((char *format, char *arg)) NO_RETURN_ATTRIBUTE;
-static void beyond __P ((void)) NO_RETURN_ATTRIBUTE;
+static void test_syntax_error PARAMS ((char *format, char *arg)) NO_RETURN_ATTRIBUTE;
+static void beyond PARAMS ((void)) NO_RETURN_ATTRIBUTE;
 
 static void
 test_syntax_error (char *format, char *arg)
@@ -373,7 +373,7 @@ term (void)
     value = binary_operator ();
 
   /* Might be a switch type argument */
-  else if ('-' == argv[pos][0] && 0 == argv[pos][2])
+  else if ('-' == argv[pos][0] && argv[pos][1] && 0 == argv[pos][2])
     {
       if (unop (argv[pos][1]))
 	value = unary_operator ();
@@ -630,7 +630,7 @@ binary_operator (void)
 static int
 unary_operator (void)
 {
-  long r, value;
+  long fd, value;
   struct stat stat_buf;
 
   switch (argv[pos][1])
@@ -787,14 +787,19 @@ unary_operator (void)
       return (TRUE == (0 != (stat_buf.st_mode & S_ISVTX)));
 #endif
 
-    case 't':			/* File (fd) is a terminal?  (fd) defaults to stdout. */
+    case 't':			/* File (fd) is a terminal? */
       advance (0);
-      if (pos < argc && isint (argv[pos], &r))
+      if (pos < argc)
 	{
+	  if (!isint (argv[pos], &fd))
+	    integer_expected_error (_("after -t"));
 	  advance (0);
-	  return (TRUE == (isatty ((int) r)));
 	}
-      return (TRUE == (isatty (1)));
+      else
+	{
+	  fd = 1;
+	}
+      return (TRUE == (isatty ((int) fd)));
 
     case 'n':			/* True if arg has some length. */
       unary_advance ();
@@ -876,14 +881,25 @@ unop (int op)
   return (member (op, "abcdefgkLhprsStuwxOGnz"));
 }
 
+static int
+one_argument (const char *s)
+{
+  if (STREQ (s, "-t"))
+    return (TRUE == (isatty (1)));
+
+  return strlen (s) != 0;
+}
+
 static int
 two_arguments (void)
 {
   int value;
 
   if (STREQ (argv[pos], "!"))
-    value = strlen (argv[pos+1]) == 0;
-  else if ((argv[pos][0] == '-') && (argv[pos][2] == '\0'))
+    value = ! one_argument (argv[pos+1]);
+  else if (argv[pos][0] == '-'
+	   && argv[pos][1] != '\0'
+	   && argv[pos][2] == '\0')
     {
       if (unop (argv[pos][1]))
 	value = unary_operator ();
@@ -932,7 +948,7 @@ posixtest (void)
 	break;
 
       case 1:
-	value = strlen (argv[1]) != 0;
+	value = one_argument (argv[1]);
 	pos = argc;
 	break;
 
@@ -962,9 +978,9 @@ posixtest (void)
 }
 
 #if defined (TEST_STANDALONE)
-#include "long-options.h"
+# include "long-options.h"
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -1035,7 +1051,7 @@ EXPRESSION is true or false and sets exit status.  It is one of:\n\
 Beware that parentheses need to be escaped (e.g., by backslashes) for shells.\n\
 INTEGER may also be -l STRING, which evaluates to the length of STRING.\n\
 "));
-      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
     }
   exit (status);
 }
@@ -1045,6 +1061,8 @@ INTEGER may also be -l STRING, which evaluates to the length of STRING.\n\
 # define main test_command
 #endif
 
+#define AUTHORS "FIXME: ksb and mjb"
+
 /*
  * [:
  *	'[' expr ']'
@@ -1076,7 +1094,8 @@ main (int margc, char **margv)
     {
       /* Don't recognize --help or --version if POSIXLY_CORRECT is set.  */
       if (getenv ("POSIXLY_CORRECT") == NULL)
-	parse_long_options (argc, argv, COMMAND_NAME, GNU_PACKAGE, VERSION, usage);
+	parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+			    AUTHORS, usage);
 
       --margc;
 
@@ -1093,7 +1112,8 @@ main (int margc, char **margv)
   if (pos >= argc)
     test_exit (SHELL_BOOLEAN (FALSE));
 
-  parse_long_options (argc, argv, COMMAND_NAME, GNU_PACKAGE, VERSION, usage);
+  parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+		      AUTHORS, usage);
   value = posixtest ();
 
   if (pos != argc)
