@@ -1,5 +1,5 @@
 /* uname -- print system information
-   Copyright (C) 89, 90, 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1989-1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,8 +41,12 @@
 #include "system.h"
 #include "error.h"
 
-static void print_element __P ((unsigned int mask, char *element));
-static void usage __P ((int status));
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "uname"
+
+#define AUTHORS "David MacKenzie"
+
+static void print_element PARAMS ((unsigned int mask, char *element));
 
 /* Values that are bitwise or'd into `toprint'. */
 /* Operating system name. */
@@ -61,7 +65,7 @@ static void usage __P ((int status));
 #define PRINT_MACHINE 16
 
  /* Host processor type. */
-# define PRINT_PROCESSOR 32
+#define PRINT_PROCESSOR 32
 
 /* Mask indicating which elements of the name to print. */
 static unsigned char toprint;
@@ -69,25 +73,45 @@ static unsigned char toprint;
 /* The name this program was run with, for error messages. */
 char *program_name;
 
-/* If nonzero, display usage information and exit.  */
-static int show_help;
-
-/* If nonzero, print the version on standard output and exit.  */
-static int show_version;
-
 static struct option const long_options[] =
 {
-  {"help", no_argument, &show_help, 1},
   {"machine", no_argument, NULL, 'm'},
   {"nodename", no_argument, NULL, 'n'},
   {"release", no_argument, NULL, 'r'},
   {"sysname", no_argument, NULL, 's'},
   {"processor", no_argument, NULL, 'p'},
-  {"version", no_argument, &show_version, 1},
   {"all", no_argument, NULL, 'a'},
+  {GETOPT_HELP_OPTION_DECL},
+  {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
 };
 
+void
+usage (int status)
+{
+  if (status != 0)
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+	     program_name);
+  else
+    {
+      printf (_("Usage: %s [OPTION]...\n"), program_name);
+      printf (_("\
+Print certain system information.  With no OPTION, same as -s.\n\
+\n\
+  -a, --all        print all information\n\
+  -m, --machine    print the machine (hardware) type\n\
+  -n, --nodename   print the machine's network node hostname\n\
+  -r, --release    print the operating system release\n\
+  -s, --sysname    print the operating system name\n\
+  -p, --processor  print the host processor type\n\
+  -v               print the operating system version\n\
+      --help       display this help and exit\n\
+      --version    output version information and exit\n"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
+    }
+  exit (status);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -102,8 +126,7 @@ main (int argc, char **argv)
 
   toprint = 0;
 
-  while ((c = getopt_long (argc, argv, "snrvpma", long_options, (int *) 0))
-	 != EOF)
+  while ((c = getopt_long (argc, argv, "snrvpma", long_options, NULL)) != -1)
     {
       switch (c)
 	{
@@ -139,20 +162,15 @@ main (int argc, char **argv)
 		     PRINT_PROCESSOR | PRINT_VERSION | PRINT_MACHINE);
 	  break;
 
+	case_GETOPT_HELP_CHAR;
+
+	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+
 	default:
 	  usage (1);
 	}
     }
 
-  if (show_version)
-    {
-      printf ("uname (%s) %s\n", GNU_PACKAGE, VERSION);
-      exit (0);
-    }
-
-  if (show_help)
-    usage (0);
-
   if (optind != argc)
     usage (1);
 
@@ -192,29 +210,3 @@ print_element (unsigned int mask, char *element)
       printf ("%s%c", element, toprint ? ' ' : '\n');
     }
 }
-
-static void
-usage (int status)
-{
-  if (status != 0)
-    fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
-  else
-    {
-      printf (_("Usage: %s [OPTION]...\n"), program_name);
-      printf (_("\
-Print certain system information.  With no OPTION, same as -s.\n\
-\n\
-  -a, --all        print all information\n\
-  -m, --machine    print the machine (hardware) type\n\
-  -n, --nodename   print the machine's network node hostname\n\
-  -r, --release    print the operating system release\n\
-  -s, --sysname    print the operating system name\n\
-  -p, --processor  print the host processor type\n\
-  -v               print the operating system version\n\
-      --help       display this help and exit\n\
-      --version    output version information and exit\n"));
-      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
-    }
-  exit (status);
-}
