@@ -1,5 +1,5 @@
 /* yes - output a string repeatedly until killed
-   Copyright (C) 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1991-1997, 1999 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,10 +25,15 @@
 #include "system.h"
 #include "long-options.h"
 
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "yes"
+
+#define AUTHORS "David MacKenzie"
+
 /* The name this program was run with. */
 char *program_name;
 
-static void
+void
 usage (int status)
 {
   if (status != 0)
@@ -42,7 +47,7 @@ Repeatedly output a line with all specified STRING(s), or `y'.\n\
 \n\
   --help      display this help and exit\n\
   --version   output version information and exit\n"));
-      puts (_("\nReport bugs to sh-utils-bugs@gnu.ai.mit.edu"));
+      puts (_("\nReport bugs to <bug-sh-utils@gnu.org>."));
     }
   exit (status);
 }
@@ -55,7 +60,10 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  parse_long_options (argc, argv, "yes", GNU_PACKAGE, VERSION, usage);
+  /* Don't recognize --help or --version if POSIXLY_CORRECT is set.  */
+  if (getenv ("POSIXLY_CORRECT") == NULL)
+    parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+			AUTHORS, usage);
 
   if (argc == 1)
     while (1)
